# $Id: factory_attr.py 2588 2009-07-28 11:03:09Z mmamonski $
#
# vim: set ts=4 sw=4 et:

#
# QosCosGrid Computing
# Copyright (C) 2008-2011 Poznan Supercomputing and Networking Center
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>
#

try: 
  from qcg.core import *
  from qcg.comp import *
except ImportError:
  class Module:
    pass
  def signal_connect(a1,a2):
    pass

from xml.dom.minidom import Document
from xml.sax import make_parser, handler
import socket
import os
import re

class SocketInputSource:
  def __init__(self, socket):
    self.socket = socket

  def getByteStream(self):
    return self

  def read(self, buf_size):
    return self.socket.recv(buf_size)

class GangliaSAXHandler(handler.ContentHandler):
  def __init__(self, doc):
    self.__QCGCompFactoryNS = "http://schemas.qoscosgrid.org/comp/2011/04/factory"
    self.hostEl = None
    self.doc = doc
    self.gangliaEl = self.doc.createElementNS(self.__QCGCompFactoryNS, "Ganglia")
    self.gangliaEl.setAttribute("xmlns", self.__QCGCompFactoryNS)
    self.doc.appendChild(self.gangliaEl)  
    if os.getenv("QCG_GANGLIA_METRIC_REGEXP") is not None:
      self.regexp = os.getenv("QCG_GANGLIA_METRIC_REGEXP")
    else:
      self.regexp = "."

  def startElement(self, name, attrs):
    if name == "HOST":
      self.hostEl = self.doc.createElementNS(self.__QCGCompFactoryNS, "H")
      self.hostEl.setAttribute("nm", attrs.getValue("NAME"))
      self.gangliaEl.appendChild(self.hostEl)

    elif name == "METRIC" and re.match(self.regexp, attrs.getValue("NAME")) is not None:
      metricEl = self.doc.createElementNS(self.__QCGCompFactoryNS, "M")
      metricEl.setAttribute("nm", attrs.getValue("NAME"))
      metricEl.appendChild(self.doc.createTextNode(attrs.getValue("VAL")))
      self.hostEl.appendChild(metricEl)
  

class ModulesInfoModule(Module):
  
  def __init__(self, *unused):
    signal_connect("smc_factory_attributes", self.signal_handler)

  def _parse_url(self, url):
    s1 = url.split("/")
    s2 = s1[2].split(":")
    return s2[0], int(s2[1])
  
  def signal_handler(self, signal, local_resource_manager_type, not_used):
    s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
    doc = Document()
    parser = make_parser()
    parser.setContentHandler(GangliaSAXHandler(doc))
    
    try:
      if os.getenv("QCG_GANGLIA_URL") == None:
        raise Exception("QCG_GANGLIA_URL env variable not set")
      
      s.connect(self._parse_url(os.getenv("QCG_GANGLIA_URL")))
      parser.parse(SocketInputSource(s))
      print str(s.recv(1024))
    finally:
        s.close()
    result = str(doc.toxml().replace("<?xml version=\"1.0\" ?>", "")) 
    doc.unlink()
    return result

#def debug(msg):
	#print msg

if __name__ == '__main__':
    _g = ModulesInfoModule()
    print _g.signal_handler(None, None, None)

