# $Id$

#
# QosCosGrid Computing
# Copyright (C) 2008-2011 Poznan Supercomputing and Networking Center
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>
#

import sys
import traceback

_torque_type = "http://schemas.qoscosgrid.org/comp/2011/04/drms/Torque"
_pbs_type = "http://schemas.qoscosgrid.org/comp/2011/04/drms/PBSPro"
_sge_type = "http://schemas.qoscosgrid.org/comp/2011/04/drms/SunGridEngine"
_slurm_type = "http://schemas.qoscosgrid.org/comp/2011/04/drms/SLURM"
_ll_type = "schemas.qoscosgrid.org/comp/2011/04/drms/Loadleveler"

# method maps
_capabilities = { }
_jobDataHandlerMap = { }
_extendedJobDataHandlerMap = { }
_queueDataHandlerMap = { }
_hostDataHandlerMap = { }
_reservationDataHandlerMap = { }

#imports
try:
    import sgeimpl
    _capabilities[_sge_type] = True
    _jobDataHandlerMap[_sge_type] = sgeimpl.getJobData
    _extendedJobDataHandlerMap[_sge_type] = None #sgeimpl.getExtendedJobData
    _queueDataHandlerMap[_sge_type] = sgeimpl.getQueueData
    _hostDataHandlerMap[_sge_type] = sgeimpl.getHostData
    _reservationDataHandlerMap[_sge_type] = sgeimpl.getReservationData
except ImportError:
    print >> sys.stderr, "Could not load SGE implementation\n", sys.exc_info()
    _capabilities[_sge_type] = False

try:
    import torqueimpl
    _capabilities[_torque_type] = True
    _jobDataHandlerMap[_torque_type] = torqueimpl.getJobData
    _extendedJobDataHandlerMap[_torque_type] = torqueimpl.addExtendedJobData
    _queueDataHandlerMap[_torque_type] = torqueimpl.getQueueData
    _hostDataHandlerMap[_torque_type] = torqueimpl.getHostData
    _reservationDataHandlerMap[_torque_type] = torqueimpl.getReservationData
except None:
    print >> sys.stderr, "Could not load Torque implementation\n", sys.exc_info()
    _capabilities[_torque_type] = False

try:
    import pbsimpl
    _capabilities[_pbs_type] = True
    _jobDataHandlerMap[_pbs_type] = pbsimpl.getJobData
    _extendedJobDataHandlerMap[_pbs_type] = pbsimpl.addExtendedJobData
    _queueDataHandlerMap[_pbs_type] = pbsimpl.getQueueData
    _hostDataHandlerMap[_pbs_type] = pbsimpl.getHostData
    _reservationDataHandlerMap[_pbs_type] = pbsimpl.getReservationData
except ImportError:
    print >> sys.stderr, "Could not load PBS Pro implementation\n", sys.exc_info()
    _capabilities[_pbs_type] = False

try:
    import slurmimpl
    _capabilities[_slurm_type] = True
    _jobDataHandlerMap[_slurm_type] = slurmimpl.getJobData
    _extendedJobDataHandlerMap[_slurm_type] = slurmimpl.addExtendedJobData
    _queueDataHandlerMap[_slurm_type] = slurmimpl.getQueueData
    _hostDataHandlerMap[_slurm_type] = slurmimpl.getHostData
    _reservationDataHandlerMap[_slurm_type] = slurmimpl.getReservationData
except ImportError:
    print >> sys.stderr, "Could not load SLURM implementation\n", sys.exc_info()
    _capabilities[_slurm_type] = False

try:
    import llimpl
    _capabilities[_ll_type] = True
    _jobDataHandlerMap[_ll_type] = llimpl.getJobData
    _extendedJobDataHandlerMap[_ll_type] = None #llimpl.getExtenedJobData
    _queueDataHandlerMap[_ll_type] = llimpl.getQueueData
    _hostDataHandlerMap[_ll_type] = llimpl.getHostData
    _reservationDataHandlerMap[_ll_type] = llimpl.getReservationData
except ImportError:
    print >> sys.stderr, "Could not load LoadLeveler implementation\n", sys.exc_info()
    _capabilities[_ll_type] = False

def _remove_version_number(system_type):
    return system_type[0:system_type.rindex("/")]

def monitoringCapabilities():
    return _capabilities.copy()

def getJobData(system_type):
    if not isinstance(system_type, str):
        raise TypeError, "system type must be a string"
    system_type = _remove_version_number(system_type);
    if system_type in _jobDataHandlerMap:
        if system_type in _capabilities:
            return _jobDataHandlerMap[system_type]()
        else:
            raise NotImplementedError, "System type: " + system_type + " is not available for query"
    else:
        raise NotImplementedError, "System type: " + system_type + " is unknown"

def addExtendedJobData(system_type, job_ids, all_job_ids):
    if not isinstance(system_type, str):
        raise TypeError, "system type must be a string"
    system_type = _remove_version_number(system_type);
    if system_type in _jobDataHandlerMap:
        if system_type in _capabilities:
            _jobDataHandlerMap[system_type]() #make sure basic job data is generated
            return _extendedJobDataHandlerMap[system_type](job_ids, all_job_ids)
        else:
            raise NotImplementedError, "System type: " + system_type + " is not available for query"
    else:
        raise NotImplementedError, "System type: " + system_type + " is unknown"


def getQueueData(system_type):
    if not isinstance(system_type, str):
        raise TypeError, "system type must be a string"
    system_type = _remove_version_number(system_type);

    if system_type in _jobDataHandlerMap:
        if system_type in _capabilities:
            return _queueDataHandlerMap[system_type]()
        else:
            raise NotImplementedError, "System type: " + system_type + " is not available for query"
    else:
        raise NotImplementedError, "System type: " + system_type + " is unknown"

def getHostData(system_type):
    if not isinstance(system_type, str):
        raise TypeError, "system type must be a string"
    system_type = _remove_version_number(system_type);
    if system_type in _jobDataHandlerMap:
        if system_type in _capabilities:
            return _hostDataHandlerMap[system_type]()
        else:
            raise NotImplementedError, "System type: " + system_type + " is not available for query"
    else:
        raise NotImplementedError, "System type: " + system_type + " is unknown"

def getReservationData(system_type):
    if not isinstance(system_type, str):
        raise TypeError, "system type must be a string"
    system_type = _remove_version_number(system_type);
    if system_type in _jobDataHandlerMap:
        if system_type in _capabilities:
            return _reservationDataHandlerMap[system_type]()
        else:
            raise NotImplementedError, "System type: " + system_type + " is not available for query"
    else:
        raise NotImplementedError, "System type: " + system_type + " is unknown"
